(ns puppetlabs.rbac-client.services.test-activity
  (:require [clojure.test :refer [deftest testing is]]
            [puppetlabs.rbac-client.services.activity :as activity]))

(deftest test-remote-activity-reporter
  (is true))
